import os
import tempfile
import uuid
import boto3
from botocore.exceptions import BotoCoreError, ClientError
from aiohttp import web

STORJ_ACCESS_KEY = os.environ.get('STORJ_ACCESS_KEY')
STORJ_SECRET_KEY = os.environ.get('STORJ_SECRET_KEY')
STORJ_END_POINT = os.environ.get('STORJ_END_POINT')
STORJ_BUCKET_NAME = os.environ.get('STORJ_BUCKET_NAME')
CDN_DOMAIN = os.environ.get('CDN_DOMAIN')


async def upload_handler(request):
    reader = await request.multipart()
    file = await reader.next()
    if file is None:
        return web.json_response({'error': 'No file uploaded'}, status=400)
    file_name, file_extension = os.path.splitext(file.filename)
    key = str(uuid.uuid4()) + file_extension
    s3 = boto3.client('s3',
                      aws_access_key_id=STORJ_ACCESS_KEY,
                      aws_secret_access_key=STORJ_SECRET_KEY,
                      endpoint_url=STORJ_END_POINT)
    try:
        with tempfile.NamedTemporaryFile() as tmp_file:
            while True:
                chunk = await file.read_chunk()  # 8192 bytes by default.
                if not chunk:
                    break
                tmp_file.write(chunk)

            s3.upload_file(
                Filename=tmp_file.name,
                Bucket=STORJ_BUCKET_NAME,
                Key=key,
                ExtraArgs={
                    "ContentType": file.content_type
                }
            )

        url = 'http://' + CDN_DOMAIN + '/' + key

        return web.json_response({'url': url, 'originalFilename': file.filename})

    except (BotoCoreError, ClientError) as e:
        print(e)
        return web.json_response({'error': 'Failed to upload file'}, status=500)


if __name__ == "__main__":
    app = web.Application()
    app.router.add_post('/upload', upload_handler)

    web.run_app(app)